<html lang="en">
<head>
  <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
  <meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1.0"/>
  <title>AIOHTTP Weather App</title>
</head>
<body>
<H1>Hello, Welcome to AIOHTTP Weather App.</H1>
</body>
</html>
