<html lang="en">
<head>
    <meta http-equiv="Content-Type" content="text/html; charset=UTF-8"/>
    <meta name="viewport"
          content="width=device-width, initial-scale=1, maximum-scale=1.0"/>
    <title>Turbogears Weather App</title>
</head>
<body>
<H1>Hello, Welcome to Turbogears <py:if test="app">${app}</py:if><py:else>Weather</py:else>App.
</H1>
<py:block name="body"></py:block>
</body>
</html>
