# Notifier Name
notifier_name = "pybrake"

# Pybrake Notifier Version
version = "1.10.1"

# End point of Airbrake API host. That will always be as default value.
AIRBRAKE_HOST = "https://api.airbrake.io"

# End point of Airbrake remote configuration.
AIRBRAKE_CONFIG_HOST = "https://notifier-configs.airbrake.io"

FLUSH_PERIOD = 15
MaxRetryAttempt = 1

HTTP_HANDLER = "http.handler"

QUEUE_HANDLER = "queue.handler"
